@@ -1,7 +1,7 @@
 #!/bin/bash
 
 # Versions available for testing via binary distributions
-OFFICIAL_RELEASES="0.8.1.1 0.8.2.2 0.9.0.0"
+OFFICIAL_RELEASES="0.8.1.1 0.8.2.2 0.9.0.1 0.10.1.1"
 
 # Useful configuration vars, with sensible defaults
 if [ -z "$SCALA_VERSION" ]; then
@@ -9,7 +9,7 @@ if [ -z "$SCALA_VERSION" ]; then
 fi
 
 # On travis CI, empty KAFKA_VERSION means skip integration tests
-# so we dont try to get binaries 
+# so we don't try to get binaries 
 # Otherwise it means test all official releases, so we get all of them!
 if [ -z "$KAFKA_VERSION" -a -z "$TRAVIS" ]; then
   KAFKA_VERSION=$OFFICIAL_RELEASES
@@ -40,6 +40,7 @@ pushd servers
         # Not sure how to construct the .tgz name accurately, so use a wildcard (ugh)
         tar xzvf $kafka/core/build/distributions/kafka_*.tgz -C ../$kafka/
         rm $kafka/core/build/distributions/kafka_*.tgz
+        rm -rf ../$kafka/kafka-bin
         mv ../$kafka/kafka_* ../$kafka/kafka-bin
       else
         echo "-------------------------------------"
@@ -51,14 +52,29 @@ pushd servers
         else
           KAFKA_ARTIFACT="kafka_${SCALA_VERSION}-${kafka}"
         fi
-        wget -N https://archive.apache.org/dist/kafka/$kafka/${KAFKA_ARTIFACT}.tgz || wget -N https://archive.apache.org/dist/kafka/$kafka/${KAFKA_ARTIFACT}.tar.gz
-        echo
-        if [ ! -d "../$kafka/kafka-bin" ]; then
-          echo "Extracting kafka binaries for ${kafka}"
+        if [ ! -f "../$kafka/kafka-bin/bin/kafka-run-class.sh" ]; then
+          echo "Downloading kafka ${kafka} tarball"
+          if hash wget 2>/dev/null; then
+            wget -N https://archive.apache.org/dist/kafka/$kafka/${KAFKA_ARTIFACT}.tgz || wget -N https://archive.apache.org/dist/kafka/$kafka/${KAFKA_ARTIFACT}.tar.gz
+          else
+            echo "wget not found... using curl"
+            if [ -f "${KAFKA_ARTIFACT}.tar.gz" ]; then
+              echo "Using cached artifact: ${KAFKA_ARTIFACT}.tar.gz"
+            else
+              curl -f https://archive.apache.org/dist/kafka/$kafka/${KAFKA_ARTIFACT}.tgz -o ${KAFKA_ARTIFACT}.tar.gz || curl -f https://archive.apache.org/dist/kafka/$kafka/${KAFKA_ARTIFACT}.tar.gz -o ${KAFKA_ARTIFACT}.tar.gz
+            fi
+          fi
+          echo
+          echo "Extracting kafka ${kafka} binaries"
           tar xzvf ${KAFKA_ARTIFACT}.t* -C ../$kafka/
+          rm -rf ../$kafka/kafka-bin
           mv ../$kafka/${KAFKA_ARTIFACT} ../$kafka/kafka-bin
+          if [ ! -f "../$kafka/kafka-bin/bin/kafka-run-class.sh" ]; then
+            echo "Extraction Failed ($kafka/kafka-bin/bin/kafka-run-class.sh does not exist)!"
+            exit 1
+          fi
         else
-          echo "$kafka/kafka-bin directory already exists -- skipping tgz extraction"
+          echo "$kafka is already installed in servers/$kafka/ -- skipping"
         fi
       fi
       echo
