@@ -1,5 +1,6 @@
 import functools
 import logging
+import operator
 import os
 import random
 import socket
@@ -10,9 +11,8 @@ import uuid
 from six.moves import xrange
 from . import unittest
 
-from kafka import KafkaClient
-from kafka.common import OffsetRequest
-from kafka.util import kafka_bytestring
+from kafka import SimpleClient
+from kafka.structs import OffsetRequestPayload
 
 __all__ = [
     'random_string',
@@ -26,15 +26,48 @@ def random_string(l):
     return "".join(random.choice(string.ascii_letters) for i in xrange(l))
 
 def kafka_versions(*versions):
+
+    def version_str_to_list(s):
+        return list(map(int, s.split('.'))) # e.g., [0, 8, 1, 1]
+
+    def construct_lambda(s):
+        if s[0].isdigit():
+            op_str = '='
+            v_str = s
+        elif s[1].isdigit():
+            op_str = s[0] # ! < > =
+            v_str = s[1:]
+        elif s[2].isdigit():
+            op_str = s[0:2] # >= <=
+            v_str = s[2:]
+        else:
+            raise ValueError('Unrecognized kafka version / operator: %s' % s)
+
+        op_map = {
+            '=': operator.eq,
+            '!': operator.ne,
+            '>': operator.gt,
+            '<': operator.lt,
+            '>=': operator.ge,
+            '<=': operator.le
+        }
+        op = op_map[op_str]
+        version = version_str_to_list(v_str)
+        return lambda a: op(version_str_to_list(a), version)
+
+    validators = map(construct_lambda, versions)
+
     def kafka_versions(func):
         @functools.wraps(func)
         def wrapper(self):
             kafka_version = os.environ.get('KAFKA_VERSION')
 
             if not kafka_version:
-                self.skipTest("no kafka version specified")
-            elif 'all' not in versions and kafka_version not in versions:
-                self.skipTest("unsupported kafka version")
+                self.skipTest("no kafka version set in KAFKA_VERSION env var")
+
+            for f in validators:
+                if not f(kafka_version):
+                    self.skipTest("unsupported kafka version")
 
             return func(self)
         return wrapper
@@ -50,22 +83,20 @@ def get_open_port():
 class KafkaIntegrationTestCase(unittest.TestCase):
     create_client = True
     topic = None
-    bytes_topic = None
     zk = None
     server = None
 
     def setUp(self):
         super(KafkaIntegrationTestCase, self).setUp()
         if not os.environ.get('KAFKA_VERSION'):
-            return
+            self.skipTest('Integration test requires KAFKA_VERSION')
 
         if not self.topic:
             topic = "%s-%s" % (self.id()[self.id().rindex(".") + 1:], random_string(10))
             self.topic = topic
-            self.bytes_topic = topic.encode('utf-8')
 
         if self.create_client:
-            self.client = KafkaClient('%s:%d' % (self.server.host, self.server.port))
+            self.client = SimpleClient('%s:%d' % (self.server.host, self.server.port))
 
         self.client.ensure_topic_exists(self.topic)
 
@@ -81,9 +112,9 @@ class KafkaIntegrationTestCase(unittest.TestCase):
 
     def current_offset(self, topic, partition):
         try:
-            offsets, = self.client.send_offset_request([ OffsetRequest(kafka_bytestring(topic), partition, -1, 1) ])
+            offsets, = self.client.send_offset_request([OffsetRequestPayload(topic, partition, -1, 1)])
         except:
-            # XXX: We've seen some UnknownErrors here and cant debug w/o server logs
+            # XXX: We've seen some UnknownErrors here and can't debug w/o server logs
             self.zk.child.dump_logs()
             self.server.child.dump_logs()
             raise
@@ -111,10 +142,3 @@ class Timer(object):
     def __exit__(self, *args):
         self.end = time.time()
         self.interval = self.end - self.start
-
-logging.basicConfig(level=logging.DEBUG)
-logging.getLogger('test.fixtures').setLevel(logging.ERROR)
-logging.getLogger('test.service').setLevel(logging.ERROR)
-
-# kafka.conn debug logging is verbose, disable in tests by default
-logging.getLogger('kafka.conn').setLevel(logging.INFO)
